/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2006                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Segment Name:  qkp000.cpp                                         */
/*                                                                   */
/* Abstract:      Member of the QKP0 DLM which comprises the         */
/*                z/TPF KPTS driver.                                 */
/*                                                                   */
/* Description:   This segment is the main entry point for the KPTS  */
/*                driver.  It will perform parsing and invoke the    */
/*                appropriate routine.                               */
/*                                                                   */
/*                This segment works with a table-driven design.     */
/*                A table entry exists for each macro/C-function,    */
/*                KPT group of APIs that is supported by this        */
/*                driver.                                            */
/*                The table entry consists of the following:         */
/*                                                                   */
/*                      - name of the KPT                            */
/*                      - pointer to assembler routine               */
/*                      - pointer to C++ function                    */
/*                                                                   */
/* ECB fields                                                        */
/* ebx000(1)  error case indicator                                   */
/* ebx001(1)  last iteration indicator                               */
/* ebcm01(1)  verbose indicator                                      */
/*                                                                   */
/*********************************************************************/
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <tpf/tpfapi.h>
#include <tpf/tpfio.h>
#include <tpf/tpfparse.h>

#include <tpf/c_pi1dt.h>
#include <c_kpts.h>

/*-------------------------------------------------------------------*/
/* make all function extern C to simplify the linkage                */
/*-------------------------------------------------------------------*/

extern "C" {
/*-------------------------------------------------------------------*/
/* structure definition                                              */
/*-------------------------------------------------------------------*/


typedef struct kpt_tbl {
            char           kpt_name[4];
            void           (*kpt_asm)(void); /* ptr to assem func    */
            void           (*kpt_c)(kptparms *);   /* ptr to C func  */
            } KPT_TBL;

const KPT_TBL  kpt_tbl[] =
{
/*    kpt name   assembler rtn    c function               */

   {  "A",       getkpta_macro,     getkpta_func },
   {  "B",       getkptb_macro,     getkptb_func },
   {  "C",       getkptc_macro,     getkptc_func },
   {  "D",       getkptd_macro,     getkptd_func },
   {  "E",       getkpte_macro,     getkpte_func },
   {  "I",       getkpti_macro,     getkpti_func },
   {  "M",       getkptm_macro,     getkptm_func },
   {  "V",       getkptv_macro,     getkptv_func },
   {  "0",       getkpt0_macro,     getkpt0_func },
   {  "1",       getkpt1_macro,     getkpt1_func },
   {  "2",       getkpt2_macro,     getkpt2_func },
/* {  "3",       getkpt3_macro,     getkpt3_func }, Customer Use */
   {  "4",       getkpt4_macro,     getkpt4_func },
/* {  "5",       getkpt5_macro,     getkpt5_func }, Customer Use */
   {  "6",       getkpt6_macro,     getkpt6_func },
   {  "9",       getkpt9_macro,     getkpt9_func },
   {  "Z",       getkptz_macro,     getkptz_func }, /* must be last */
   {  "", NULL, NULL }};

/*********************************************************************/
/*                                                                   */
/* QKPT( )                                                           */
/*                                                                   */
/* Entry point to the ZTEST KPTS  functional message.                */
/*                                                                   */
/*********************************************************************/
void QKP0(void)
{
   int                  i;
   int                  rc;
   char                 kpt[4] = { 0 };
   char                 help[255] = { 0 };
   char                 spaces[5] = { 0x40 };
   char                *buffer_ptr;
   boolean              found = FALSE;
   int                  count = 1;
   struct pi1dt        *pidt;


   kptparms *kptparms_p;

   kptparms_p = (kptparms *)malloc(sizeof(kptparms));
   memcpy(&(ecbptr()->ebx008),&kptparms_p,sizeof(kptparms_p));

   ecbptr()->ebrout = 0x010000;
   kptparms_p->error_case  = 0;           /*default:  error case = 0      */
   (kptparms_p->tout)    = FALSE;           /*default:  no timeout tests    */
   (kptparms_p->procreq) = FALSE;           /*default:  use local proc ord  */
   (kptparms_p->ksareq)  = FALSE;
   (kptparms_p->klgfbareq)= FALSE;
   (kptparms_p->hdrsreq) = FALSE;
   (kptparms_p->cso)     = TRUE;            /*default:  Do CSO tests        */
   (kptparms_p->bso)     = TRUE;            /*default:  Do BSO tests        */
   (kptparms_p->kbareq)  = FALSE;
   (kptparms_p->refresh) = FALSE;
   (kptparms_p->snap)    = FALSE;            /*default:  no snaps            */
   (kptparms_p->image)   = NULL;             /*default:  current image       */
   (kptparms_p->kpt)     = NULL;
   (kptparms_p->verb)    = FALSE;            /*default: noverbose            */
   (kptparms_p->tag)     = FALSE;            /*default: no tagging           */


   ecbptr()->ebcm01 = FALSE;               /*default:  verbose=no          */

   pidt = (struct pi1dt *) cinfc_fast(CINFC_CMMPID);
   kptparms_p->cpuord=pidt->pi1pids.pi1cidn;
   kptparms_p->cpuid=pidt->pi1pids.pi1cpui;

   if ( getkpt(kpt, kptparms_p, &(ecbptr()->ebcm01)) == FALSE )
   {
/*********************************************************************************/
/*                           help display                                        */
      sprintf(help, "\n FORMAT: ZTEST KPTS KPT-x \n"
              "         (NO)Snap (NO)TOut (NO)VERBose (NO)Cso (NO)Bso PROC-proc \n"
              "         IMage-image (NO)KSA (NO)KBA (NO)KLGFBA (NO)REFRESH \n"
              "         (NO)VERIFY (NO)HDRS ERRorcase-d \n");
      rc = wtopc(help, WTOPC_EBROUT, WTOPC_PAGE, WTOPC_NO_HEADER);
      if ( rc == WTOPC_PAGE_STOP )
         exit(0);

      memset (help, 0x00, sizeof(help));
      sprintf(help," KPT-x is the only required parameter, specify ALL or a specific KPT. \n"
              " \n The following parameters are optional: \n");
      rc = wtopc(help, WTOPC_EBROUT, WTOPC_PAGE, WTOPC_NO_HEADER);
      if ( rc == WTOPC_PAGE_STOP )
         exit(0);

      memset (help, 0x00, sizeof(help));
      sprintf(help," NOSnap is the default, specify Snap to display the first bytes of the kpt. \n"
              " NOTOut is the default, specify TOut to exercise the timeout feature. \n"
              " NOVERBose is the default, specify VERBose to display detailed messages.");
      rc = wtopc(help, WTOPC_EBROUT, WTOPC_PAGE, WTOPC_NO_HEADER);
      if ( rc == WTOPC_PAGE_STOP )
         exit(0);

      memset (help, 0x00, sizeof(help));
      sprintf(help," Cso is the default, specify NOCso to skip CSO tests. \n"
              " Bso is the default, specify NOBso to skip BSO tests. \n"
              " PROC-proc is to specify a processor ID different from the current processor.");
      rc = wtopc(help, WTOPC_EBROUT, WTOPC_PAGE, WTOPC_NO_HEADER);
      if ( rc == WTOPC_PAGE_STOP )
         exit(0);

      memset (help, 0x00, sizeof(help));
      sprintf(help," IMage-image is to specify image 1-8 of CTKX. \n"
              " NOKSA is the default, specify KSA to use the keypoint staging area. \n"
              " NOKBA is the default, specify KBA to use the keypoint backup area.");
      rc = wtopc(help, WTOPC_EBROUT, WTOPC_PAGE, WTOPC_NO_HEADER);
      if ( rc == WTOPC_PAGE_STOP )
         exit(0);

      memset (help, 0x00, sizeof(help));
      sprintf(help," NOKLFGBA is the default, specify KLGFBA to use the LGF backup area. \n"
              " NOREFRESH is the default, specify REFRESH to refresh core copy. \n"
              " NOVERIFY is the default, specify VERIFY to verify keypoint fetch.");
      rc = wtopc(help, WTOPC_EBROUT, WTOPC_PAGE, WTOPC_NO_HEADER);
      if ( rc == WTOPC_PAGE_STOP )
         exit(0);

      memset (help, 0x00, sizeof(help));
      sprintf(help," HDRS is the default, specify NOHDRS to strip keypoint headers. \n"
              " ERRorcase-d is to specify the error case to test.");
      rc = wtopc(help, WTOPC_EBROUT, WTOPC_PAGE, WTOPC_NO_HEADER);
      if ( rc == WTOPC_PAGE_STOP )
         exit(0);

      memset (help, 0x00, sizeof(help));
      sprintf(help,"\n The following are valid values for KPT. \n" );
      rc = wtopc(help, WTOPC_EBROUT, WTOPC_PAGE, WTOPC_NO_HEADER);
      if ( rc == WTOPC_PAGE_STOP )
         exit(0);
      memset (spaces,' ', sizeof(spaces));
      count = 0;
      memset (help, 0x00, sizeof(help));
      buffer_ptr = help;
      for ( i = 0; memcmp(kpt_tbl[i].kpt_name, "00", 1); i++ )
      {
         if ( buffer_ptr >= (help + sizeof(help) - sizeof(kpt) - sizeof(spaces) -count - 2 ))
         {
            rc = wtopc(help, WTOPC_EBROUT, WTOPC_PAGE, WTOPC_NO_HEADER);
            if ( rc == WTOPC_PAGE_STOP )
               exit(0);
            memset (help, 0x00, sizeof(help));
            count = 0;
            buffer_ptr = help;
         }
         buffer_ptr += sprintf (buffer_ptr, " %s%.5s",
                                kpt_tbl[i].kpt_name,
                                spaces);
         ++count;
         if ( count == 4 )
         {
            memcpy(buffer_ptr, "\n", 1);
            ++buffer_ptr;
            count = 0;
         }
      }
      rc = wtopc(help, WTOPC_EBROUT, WTOPC_PAGE, WTOPC_NO_HEADER);
      if ( rc == WTOPC_PAGE_STOP )
         exit(0);
      wtopc("End of KPTS help display", WTOPC_EBROUT, WTOPC_PAGE_END, WTOPC_NO_HEADER);
      exit(0);
   }
   /*********************************************************************************/


   if ( memcmp(kpt, "ALL", 3) == 0 )    /* execute cases for all KPTs except KPT-Z*/
   {

       if (kptparms_p->error_case > 0)
       {
          prtmsg("Error cases can only be executed for individual kpts \n");
          exit(0);
       }
       else
          prtmsg("executing normal cases for all KPTs \n");
       while (count > 0)
       {
           for ( i = 0; memcmp(kpt_tbl[i].kpt_name, "00", 1); i++ )
           {
              if ( memcmp(kpt_tbl[i].kpt_name,"Z",1) != 0)
              {
                 defrc();
                 if (kptparms_p->bso)
                 {
                   if ( kpt_tbl[i].kpt_asm )
                   {
                       memcpy(&(ecbptr()->ebx008),&kptparms_p,sizeof(kptparms_p));
                      (*kpt_tbl[i].kpt_asm)();
                   }
                 }
                 if (kptparms_p->cso)
                 {
                   if ( kpt_tbl[i].kpt_c )
                   {
                      (*kpt_tbl[i].kpt_c)(kptparms_p);
                   }
                 }
              }
           }

           count--;

       }

       found = TRUE;
    }
    else
    {                           /* execute specific KPT test cases   */
       for ( i = 0; memcmp(kpt_tbl[i].kpt_name, "00", 1); i++ )
       {
          if ( memcmp(kpt, kpt_tbl[i].kpt_name, strlen(kpt)) == 0)
          {
              found = TRUE;

              while (count > 0)
              {
                  /*  if this is the only or last iteration, set the indicator */
                  if ( count == 1 )
                     (ecbptr()->ebx001) = TRUE;
                  defrc();
                  if (kptparms_p->bso)
                  {
                    if ( kpt_tbl[i].kpt_asm )
                    {
                       (*kpt_tbl[i].kpt_asm)();
                    }
                  }
                  if (kptparms_p->cso)
                  {
                    if ( kpt_tbl[i].kpt_c )
                    {
                       (*kpt_tbl[i].kpt_c)(kptparms_p);
                    }
                  }

                  count--;

              }
              break;

          } /* end if the matching KPT is found */
       }  /* end for loop that searches specific KPT */

       if ( ! found )
       {
          prtmsg(kpt);
          prtmsg(":  is not in the KPT table, no test cased executed \n");
       }
    } /* end executing specific KPT test cases */

   printf("KPTS driver exited \n");

   exit(0);
}


/*********************************************************************/
/*                                                                   */
/* getkpt( )                                                         */
/*                                                                   */
/* Routine to parse the input functional message and extract the     */
/* name of the kpt   which is to be executed and the error case      */
/* number if supplied.                                               */
/*                                                                   */
/*********************************************************************/
boolean getkpt(char *kpt, kptparms *kptparms_p, unsigned char *verbose )
{
   int    i;
   struct IPRSE_output  parsed_data, *pparm;
   struct mi0mi        *blk_ptr;
   char                *input_ptr, *eom_ptr;
   int                  parser_return;

   blk_ptr = (struct mi0mi *)ecbptr()->ce1cr0;
   input_ptr = blk_ptr->mi0acc+strlen("Zxxxx");
   eom_ptr = (char *)&blk_ptr->mi0ln0 + blk_ptr->mi0cct - 1;
   *eom_ptr = '\0';

   unsigned long current_image = 0;

   parser_return = IPRSE_parse(input_ptr,
                              QKPT_GRAMMAR,
                              &parsed_data,
                              IPRSE_ALLOC+IPRSE_PRINT,
                              "QKPT");

   /******************************************************************/
   /* return -1 indicates HELP or ?                                  */
   /* return  0 indicates message did not meet grammar requirements  */
   /******************************************************************/

   if ( parser_return == -1 )
   {
      return(FALSE);
   }
   if ( parser_return == 0 )
   {
      return(FALSE);
   }

   /******************************************************************/
   /* Else, determine which parameters were input and set the        */
   /* appropriate values to return to the caller so the corresponding*/
   /* KPTs can be tested.                                            */
   /******************************************************************/
   for ( i = 1, pparm = &parsed_data;
         i <= parser_return;
         i++ , pparm = pparm->IPRSE_next )
   {
      if ( memcmp(pparm->IPRSE_parameter, "KPT", 3) == 0 )
      {
         memcpy(kpt, pparm->IPRSE_value, strlen(pparm->IPRSE_value));
      }
      else if ( strcmp(pparm->IPRSE_parameter, "(NO)TOut") == 0 )
           {
               kptparms_p->tout =( *(pparm->IPRSE_value)) == 'Y' ? TRUE : FALSE;
           }
      else if ( strcmp(pparm->IPRSE_parameter, "(NO)Cso") == 0 )
           {
               if ((*(pparm->IPRSE_value)) == 'Y')
               {
                   kptparms_p->bso = FALSE;
               }
               kptparms_p->cso = *(pparm->IPRSE_value) == 'Y' ? TRUE : FALSE;
           }
      else if ( strcmp(pparm->IPRSE_parameter, "(NO)Bso") == 0 )
           {
               if (*(pparm->IPRSE_value) == 'Y')
               {
                   kptparms_p->cso = FALSE;
               }
               kptparms_p->bso = *(pparm->IPRSE_value) == 'Y' ? TRUE : FALSE;
           }
      else if ( strcmp(pparm->IPRSE_parameter, "(NO)Snap") == 0 )
           {
               kptparms_p->snap = *(pparm->IPRSE_value) == 'Y' ? TRUE : FALSE;
           }
      else if (  memcmp(pparm->IPRSE_parameter, "PROC", 4) == 0 )
           {
               kptparms_p->cpuid = *(pparm->IPRSE_value);
               kptparms_p->procreq=TRUE;
               kptparms_p->cpuord = cpuNameToOrdinal(*(pparm->IPRSE_value));
               if ((kptparms_p->cpuord) == -1)
               {
                    prtmsg("Invalid processor ID \n");
                    return(FALSE);
               }
           }
      else if ( strcmp(pparm->IPRSE_parameter, "(NO)KBA") == 0 )
           {
               kptparms_p->kbareq = (*(pparm->IPRSE_value)) == 'Y' ? TRUE : FALSE;
           }
      else if ( strcmp(pparm->IPRSE_parameter, "(NO)KSA") == 0 )
           {
               kptparms_p->ksareq =( *(pparm->IPRSE_value)) == 'Y' ? TRUE : FALSE;
               if (kptparms_p->ksareq==TRUE)
                 current_image= (unsigned long)cinfc_fast(CINFC_CMMAIM);
                 kptparms_p->image = current_image;

           }
      else if ( strcmp(pparm->IPRSE_parameter, "(NO)KLGFBA") == 0 )
           {
               kptparms_p->klgfbareq = *(pparm->IPRSE_value) == 'Y' ? TRUE : FALSE;
           }
      else if ( strcmp(pparm->IPRSE_parameter, "(NO)HDRS") == 0 )
           {
               kptparms_p->hdrsreq = *(pparm->IPRSE_value) == 'Y' ? TRUE : FALSE;
           }
      else if ( strcmp(pparm->IPRSE_parameter, "(NO)REFRESH") == 0 )
           {
               kptparms_p->refresh = *(pparm->IPRSE_value) == 'Y' ? TRUE : FALSE;
           }

      else if ( strcmp(pparm->IPRSE_parameter, "(NO)VERIFY") == 0 )
           {
               kptparms_p->tag = *(pparm->IPRSE_value) == 'Y' ? TRUE : FALSE;
           }

      else if ( strcmp(pparm->IPRSE_parameter, "(NO)VERBose") == 0 )
           {
               *verbose = *(pparm->IPRSE_value) == 'Y' ? TRUE : FALSE;
               kptparms_p->verb = *(pparm->IPRSE_value) == 'Y' ? TRUE : FALSE;

           }

      else if ( memcmp(pparm->IPRSE_parameter, "IMage", 5) == 0 )
           {
               kptparms_p->image = atoi(pparm->IPRSE_value);
               if(kptparms_p->image < 1 | kptparms_p->image > 9)
               {
                    prtmsg("Invalid image number \n");
                    return(FALSE);
               }
           }
      else if ( memcmp(pparm->IPRSE_parameter, "ERRorcase", 9) == 0 )
           {
              kptparms_p->error_case = atoi(pparm->IPRSE_value);
              if(kptparms_p->error_case < 1 | kptparms_p->error_case > 9)
              {
                    prtmsg("Invalid error case number \n");
                    return(FALSE);
              }
           }
   }
   return(TRUE);
}

short cpuNameToOrdinal(const char Name)
{
  char  CPUName = Name;
  short CPUOrdinal = -1;     // error return
/*

  PIDT_CPU_NAME_TO_ORDINAL(&CPUOrdinal, &CPUName);
*/
  cvt_cpuid(&CPUName,&CPUOrdinal);

  return CPUOrdinal;
}


} /* end of extern "C"                                               */
